from distutils.core import setup


setup(
    name="hotreloader",
    packages=["."],
    version="0.1.0",
    description="A hot-reload framework can watch files and handle it when file is edited.",
    url="https://github.com/fswair/PyReloader",
    author="Mert Sırakaya",
    author_email="usirakaya@ogr.iu.edu.tr",
    keywords=["hot-reload", "file-watching", "reloader"],
    classifiers=[]
)